/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef BEE_H
#define BEE_H

#include "Colony.h"
#include "Flower.h"
#include "FlowerLocator.h"
#include "Position.h"
#include "RunSettings.h"
#include <vector>

class Bee
{
    public:
        Bee(const unsigned id, Position *, beeType, Colony *, unsigned inputWorldSize);
        ~Bee();
        void updateBee(FlowerLocator *flowerLocations);

        const Position *getPosition(void);
        const Position *getRealKnownFlowerPosition(void);
        const beeType getRole(void);
        const unsigned getId(void);
        const bool hasNectar(void);
        const unsigned getForgetFlowerThreshold(void) const;
        const unsigned getDanceFlowerThreshold(void) const;

    protected:
    private:
        unsigned myId;
        unsigned worldSize;

        int flightEnergyLeft;
        int flightEnergyUsed;
        bool carryingNectar;

        //movement data
        Position *previousDestination;
        bool thinkWeReachedDestination;

        //scout movement data
        Position *desiredPosition;
        float velocity;
        unsigned bearing;
        bool returnHome;

        //if we think we should have been home, but aren't yet
        bool thinkAtHome;
        bool newRandomPosition;
        bool randomWalking;
        bool arrivedAtFlowerLocation;

        bool hasDanced;
        bool dancing;
        unsigned danceTimeLeft;

        beeType myRole;

        Position *knownFlowerPosition;
        //do not use in real simulation--just used for evaluation
        Position *realKnownFlowerPosition;
        float knownFlowerQuality;
        bool hasKnownFlower;
        int nectarQuantity;

	bool runAccounted;
	bool doDanceNow;
	
        float forgetFlowerThreshold;
        float danceFlowerThreshold;

        Position *myPosition;
        Position *perceivedPosition;
        Colony *myColony;

        void moveBee(void);
        void hone(const Position *);
        void scoutMove(void);
        void randomWalk(void);
        void goHome(void);

        void detectFlower();
        bool findAndHarvestFlower();
        float getKnownFlowerValue();

        float getFlowerValue(const Position *position, float flowerQuality, float quantity);


        void getNewSpiralDestination(void);
};

#endif // BEE_H
