/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Colony.h"
#include "RunSettings.h"
#include <assert.h>
#include <iostream>
#include <algorithm>
#include <stdlib.h>

using namespace std;

Colony::Colony(Position *colonyPosition)
{
    myPosition = colonyPosition;
    numFood = 0;
    totalEnergy = 0;
    dancesFollowed = 0;
    numScouts = 0;
    dancesOffered = 0;
}

Colony::~Colony()
{
    delete myPosition;
}

const Position *Colony::getPosition()
{
    return myPosition;
}

/*
    Allow each dancer to be used exactly n times per timestep
    Call this function each timestep to do so.
*/
void Colony::initializeDancers(void)
{
    dancersToConsider.clear();
    vector<Position *>::iterator dancer;

    for(dancer = dancers.begin(); dancer != dancers.end(); dancer++)
    {
        dancersToConsider.push_back(pair<Position *, unsigned>(*dancer, 0));
    }
}

void Colony::startDance(Position *dancer)
{
    assert(dancer);

    dancesOffered++;

    dancers.push_back(dancer);
    dancersToConsider.push_back(pair<Position *, unsigned>(dancer, 0));
}

void Colony::endDance(Position *dancer)
{
    vector<Position *>::iterator dancerFound = find(dancers.begin(), dancers.end(), dancer);
    assert(dancerFound != dancers.end());
    dancers.erase(dancerFound);

    vector<pair<Position *, unsigned> >::iterator dancerToConsider;
    for(dancerToConsider = dancersToConsider.begin(); dancerToConsider != dancersToConsider.end(); dancerToConsider++)
    {
        if(*((*dancerToConsider)).first == *dancer)
        {
            dancersToConsider.erase(dancerToConsider);
            break;
        }
    }
}

void Colony::deliverFood(unsigned quantity)
{
    assert(quantity > 0);

    numFood += quantity;
}

const unsigned Colony::getEnergy(const unsigned used)
{
    totalEnergy += used;

    return runSettings->getEnergyToReplenish();
}

Position * Colony::getFlowerAssignment(void)
{
    unsigned availableFlowers = dancersToConsider.size();

    if (!availableFlowers)
    {
        return NULL;
    }

    dancesFollowed++;

    unsigned chosenIndex = rand() % availableFlowers;
    Position *chosen = dancersToConsider[chosenIndex].first;
    (dancersToConsider[chosenIndex].second)++;

    if(dancersToConsider[chosenIndex].second == runSettings->getNumFollowersPerDancer())
    {
        dancersToConsider.erase(dancersToConsider.begin() + chosenIndex);
    }

    return chosen;
}

unsigned Colony::getEnergySpent(void)
{
    return totalEnergy;
}

unsigned Colony::getDancesFollowed(void)
{
    return dancesFollowed;
}

unsigned Colony::getDancesOffered(void)
{
    return dancesOffered;
}

unsigned Colony::getNectarCollected(void)
{
    return numFood;
}

unsigned Colony::getNumScouts(void)
{
    return numScouts;
}

void Colony::reportScout(void)
{
    numScouts++;
}
