/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef COLONY_H
#define COLONY_H

#include "Flower.h"
#include "Position.h"
#include <vector>

class Colony
{
    public:
        Colony(Position *);
        ~Colony();

        void initializeDancers();

        Position const *getPosition();
        void deliverFood(unsigned);
        const unsigned getEnergy(const unsigned used);

        Position *getFlowerAssignment(void);

        void startDance(Position *);
        void endDance(Position *);
        unsigned getEnergySpent(void);
        unsigned getNectarCollected(void);
        unsigned getDancesFollowed(void);
        unsigned getDancesOffered(void);
        unsigned getNumScouts(void);
        void reportScout(void);

    private:
        Position *myPosition;
        unsigned numFood;
        unsigned totalEnergy;
        unsigned dancesFollowed;
        unsigned dancesOffered;
        unsigned numScouts;
        std::vector<Position *> dancers;
        std::vector<std::pair<Position *, unsigned> > dancersToConsider;

};

#endif // COLONY_H
