/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CONFIGREADER_H
#define CONFIGREADER_H

#include <fstream>
#include "Settings.h"
#include "World.h"

class ConfigReader
{
    public:
        ConfigReader(char *inputFile);
        ~ConfigReader();

        Settings *getSettings(void);
        World *getWorld(void);

    private:
        std::ifstream configFile;
        std::ifstream worldFile;
        unsigned timeSteps;
        unsigned nectarPerHarvest;
        unsigned beeFlightSpeed;
        unsigned energyToReplenish;
        float locationAccuracy;
        float spiralVelocityIncrease;
        float spiralAngularVelocityIncrease;
        float beeVisionRadius;
        float actuationNoise;
        float sensingNoise;
        float unemployedScoutProbability;
        unsigned sharePositionalInformation;
        unsigned beeDanceLengthMultiplier;
        float beeColonySightRadius;
        unsigned numFollowersPerDancer;
        int randSeed;
        scoutStrategy chosenStrategy;

        char logFile[100];


        void readConfig(void);
};

#endif // CONFIGREADER_H
