/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <assert.h>
#include "Default.h"
#include "Flower.h"

using namespace std;

Flower::Flower(const unsigned id,
               Position *inputPosition,
               unsigned lifespan,
               unsigned initialQuantity,
               unsigned inputQuality)
{
    myPosition = inputPosition;
    nectarRemaining = initialQuantity;
    ageLeft = lifespan;
    myId = id;
    quality = inputQuality;

    isLoggedAsDead = false;
}

Flower::~Flower()
{
    delete myPosition;
}

const unsigned Flower::getId()
{
    return myId;
}

const Position *Flower::getPosition() const
{
    return myPosition;
}

const unsigned Flower::harvestNectar()
{
    if(!this->isDead())
    {
        nectarRemaining -= DEFAULT_NECTAR_PER_HARVEST;
        return true;
    }
    else
    {
        return false;
    }
}

void Flower::updateFlower()
{
    ageLeft -= 1;
}

const bool Flower::isDead()
{
    return nectarRemaining <= 0;//(ageLeft <= 0) or (nectarRemaining <= 0);
}

const bool Flower::canDelete()
{
    return this->isDead();
}

const bool Flower::shouldMarkDead(void)
{
    if(!this->isDead() || isLoggedAsDead)
        return false;

    isLoggedAsDead = true;
    return true;
}

const unsigned Flower::getQuality(void) const
{
    return quality;
}

const int Flower::getQuantity(void) const
{
    return nectarRemaining;
}
