/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "FlowerLocator.h"

#include <algorithm>
#include <assert.h>
#include <iostream>

using namespace std;

FlowerLocator::FlowerLocator()
{
    flowerLocations = new map<pair<int, int>, vector<Flower *> *>();
}

FlowerLocator::~FlowerLocator()
{
    map<pair<int, int>, vector<Flower *> *>::iterator vectorIterator, deleteIterator;
    vectorIterator = flowerLocations->begin();

    while(flowerLocations->begin() != flowerLocations->end())
    {
        deleteIterator = vectorIterator++;

        if(deleteIterator != flowerLocations->end())
            delete(deleteIterator->second);
        else
            break;
    }

    delete flowerLocations;
}

void FlowerLocator::insertFlower(Flower *toAdd)
{
    map<pair<int, int>, vector<Flower *> *>::iterator found;
    const Position *flowerPosition = toAdd->getPosition();
    pair<int, int> flowerPair = pair<int, int>((int) flowerPosition->getXCoord(),
                                               (int) flowerPosition->getYCoord());

    found = flowerLocations->find(flowerPair);

    if(found != flowerLocations->end())
    {
        found->second->push_back(toAdd);
    }
    else
    {
        (*flowerLocations)[flowerPair] = new vector<Flower *> (1, toAdd);
    }
}

void FlowerLocator::deleteFlower(Flower *toRemove)
{
    map<pair<int, int>, vector<Flower *> *>::iterator found;
    const Position *flowerPosition = toRemove->getPosition();
    vector<Flower *>::iterator flowerVectorIterator;
    pair<int, int> flowerPair = pair<int, int>((int) flowerPosition->getXCoord(),
                                               (int) flowerPosition->getYCoord());

    assert(toRemove->isDead());

    found = flowerLocations->find(flowerPair);

    if(found != flowerLocations->end())
    {
        flowerVectorIterator = find(found->second->begin(),
                                    found->second->end(),
                                    toRemove);

        if(flowerVectorIterator != found->second->end())
        {
            found->second->erase(flowerVectorIterator);
        }

        if(found->second->empty())
        {
            flowerLocations->erase(flowerPair);
        }
    }
}

vector<Flower *> * FlowerLocator::findFlowersByPosition(const Position *position, float radius) const
{
    map<pair<int, int>, vector<Flower *> *>::iterator found;
    vector<Flower *>::iterator flowerVectorIterator;

    vector<Flower *> *ret = NULL;

    float curXPos = position->getXCoord();
    float curYPos = position->getYCoord();

    for(float xPos = curXPos-radius-.5; xPos <= curXPos+radius+.5; xPos++)
    {
        for(float yPos = curYPos-radius-.5; yPos <= curYPos+radius+.5; yPos++)
        {
            pair<int, int> positionPair = pair<int, int>((int) xPos,
                                                         (int) yPos);

            found = flowerLocations->find(positionPair);

            if(found != flowerLocations->end())
            {
                for(flowerVectorIterator = found->second->begin();
                    flowerVectorIterator != found->second->end();
                    flowerVectorIterator++)
                {
                    //if we're looking for a flower within one unit (radius 0), then don't worry about radius checking
                    if(!radius || (*flowerVectorIterator)->getPosition()->getDistance(*position) < radius)
                    {
                        if(!ret)
                            ret = new vector<Flower *>();

                        ret->push_back(*flowerVectorIterator);
                    }
                }
            }
        }
    }

    return ret;
}
