/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Default.h"

#include <iostream>
#include "Default.h"
#include "Logger.h"
#include "RunSettings.h"

using namespace std;

Logger::Logger()
{
    outFile.open(runSettings->getLogFileName());
    outFile << LOGDOCSTRING;
}

Logger::~Logger()
{
    outFile.close();
}

void Logger::logBee(Bee *bee)
{
    const Position *beePos = bee->getPosition();

    outFile << "B " << bee->getId() << " ";

    if(bee->getRole() == SCOUT)
    {
        outFile << "S ";
    }
    else if(bee->hasNectar())
    {
        outFile << "F+ ";
    }
    else
    {
        outFile << "F ";
    }

    outFile << beePos->getXCoord() << " " << beePos->getYCoord()
            << endl;
}

void Logger::logFlower(Flower *flower)
{
    const Position *flowerPos = flower->getPosition();

    outFile << "F " << flower->getId() << " ";

    if(flower->isDead())
    {
        outFile << "D " << endl;
    }
    else
    {
        outFile << flowerPos->getXCoord() << " " << flowerPos->getYCoord()
        << " " << flower->getQuality() << endl;
    }
}

void Logger::logTimeStep(unsigned time)
{
    outFile << "T " << time << endl;
}

void Logger::logWorldStats(unsigned worldSize)
{
    outFile << "W " << worldSize << endl;
}

void Logger::logFinalStats(unsigned numFood, unsigned totalEnergy)
{
    outFile << "END: " << numFood << " food, "
            << totalEnergy << " energy" << endl;
}
