/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <float.h>
#include <cmath>
#include "Position.h"
#include "RunSettings.h"

Position::Position(float x, float y)
{
    xCoord = x;
    yCoord = y;
}

Position::~Position()
{
    //dtor
}

float Position::getXCoord() const
{
    return xCoord;
}

float Position::getYCoord() const
{
    return yCoord;
}

void Position::setXCoord(float newX)
{
    xCoord = newX;
}

void Position::setYCoord(float newY)
{
    yCoord = newY;
}

void Position::incrementXCoord(float diff)
{
    xCoord += diff;
}

void Position::incrementYCoord(float diff)
{
    yCoord += diff;
}

float Position::getDistance(const Position &toCompare) const
{
  return sqrt(pow(fabs(xCoord - toCompare.getXCoord()), 2) +
	      pow(fabs(yCoord - toCompare.getYCoord()), 2));
}

float Position::getAbsoluteSlope(const Position&toCompare) const
{
    float run = xCoord - toCompare.getXCoord();

    if(run == 0)
        return FLT_MAX;

    return fabs(yCoord - toCompare.getYCoord())/
           fabs(xCoord - toCompare.getXCoord());
}

float Position::getAngle(const Position &toCompare) const
{
    return atan2((toCompare.getYCoord()-yCoord),
                 (toCompare.getXCoord()-xCoord));
}

bool Position::operator==(const Position &other) const
{
    return (this->getDistance(other) <= runSettings->getLocationAccuracy());
}
