/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>
#include "Settings.h"

Settings::Settings(unsigned inputTimeSteps,
                   unsigned inputNectarPerHarvest,
                   unsigned inputBeeFlightSpeed,
                   unsigned inputEnergyToReplenish,
                   float inputLocationAccuracy,
                   int inputRandSeed,
                   scoutStrategy inputChosenStrategy,
                   float inputSpiralVelocityIncrease,
                   float inputBeeVisionRadius,
                   float inputActuationNoise,
                   float inputSensingNoise,
                   float inputBeeColonySightRadius,
                   float inputUnemployedScoutProbability,
                   unsigned inputAngularVelocityIncrease,
                   unsigned inputBeeDanceLengthMultiplier,
                   unsigned inputSharePositionalInformation,
                   unsigned inputNumFollowersPerDancer,
                   char *inputLogFileName)
{
    beeDanceLengthMultiplier = inputBeeDanceLengthMultiplier;
    unemployedScoutProbability = inputUnemployedScoutProbability;
    beeColonySightRadius = inputBeeColonySightRadius;
    timeSteps = inputTimeSteps;
    nectarPerHarvest = inputNectarPerHarvest;
    beeFlightSpeed = inputBeeFlightSpeed;
    energyToReplenish = inputEnergyToReplenish;
    locationAccuracy = inputLocationAccuracy;
    randSeed = inputRandSeed;
    chosenScoutStrategy = inputChosenStrategy;
    spiralVelocityIncrease = inputSpiralVelocityIncrease;
    spiralAngularVelocityIncrease = inputAngularVelocityIncrease;
    beeVisionRadius = inputBeeVisionRadius;
    beeActuationError = inputActuationNoise;
    beeSensingError = inputSensingNoise;
    sharePositionalInformation = inputSharePositionalInformation;
    numFollowersPerDancer = inputNumFollowersPerDancer;
    strncpy(logFileName, inputLogFileName, 50);
}

const unsigned Settings::getTimeSteps() const
{
    return timeSteps;
}

const unsigned Settings::getNectarPerHarvest() const
{
    return nectarPerHarvest;
}

const unsigned Settings::getBeeFlightSpeed() const
{
    return beeFlightSpeed;
}

const unsigned Settings::getBeeDanceLengthMultiplier() const
{
    return beeDanceLengthMultiplier;
}

const unsigned Settings::getEnergyToReplenish() const
{
    return energyToReplenish;
}

const float Settings::getLocationAccuracy() const
{
    return locationAccuracy;
}

const int Settings::getRandSeed() const
{
    return randSeed;
}

const scoutStrategy Settings::getChosenScoutStrategy() const
{
    return chosenScoutStrategy;
}

const char *Settings::getLogFileName() const
{
    return logFileName;
}

const float Settings::getSpiralVelocityIncrease() const
{
    return spiralVelocityIncrease;
}

const unsigned Settings::getSpiralAngularVelocityIncrease() const
{
    return spiralAngularVelocityIncrease;
}

const float Settings::getBeeVisionRadius() const
{
    return beeVisionRadius;
}

const float Settings::getBeeColonySightRadius() const
{
    return beeColonySightRadius;
}

const float Settings::getUnemployedScoutProbability() const
{
    return unemployedScoutProbability;
}

const float Settings::getSensingNoise() const
{
    return beeSensingError;
}

const float Settings::getActuationNoise() const
{
    return beeActuationError;
}

const unsigned Settings::doSharePositionalInformation() const
{
    return sharePositionalInformation;
}

const unsigned Settings::getNumFollowersPerDancer() const
{
    return numFollowersPerDancer;
}
