/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SETTINGS_H
#define SETTINGS_H

enum beeType { FORAGER, SCOUT };
enum scoutStrategy { RANDOMWALK, SPIRAL };

class Settings
{
    public:
        Settings(unsigned inputTimeSteps,
                unsigned inputNectarPerHarvest,
                unsigned inputBeeFlightSpeed,
                unsigned inputEnergyToReplenish,
                float inputLocationAccuracy,
                int inputRandSeed,
                scoutStrategy inputChosenStrategy,
                float inputSpiralVelocityIncrease,
                float inputBeeVisionRadius,
                float inputActuationNoise,
                float inputSensingNoise,
                float inputBeeColonySightRadius,
                float inputUnemployedScoutProbability,
                unsigned inputAngularVelocityIncrease,
                unsigned inputBeeDanceLengthMultiplier,
                unsigned inputSharePositionalInformation,
                unsigned inputNumFollowersPerDancer,
                char *inputLogFileName);

        const unsigned getTimeSteps() const;
        const unsigned getNectarPerHarvest() const;
        const unsigned getBeeFlightSpeed() const;
        const unsigned getEnergyToReplenish() const;
        const float getLocationAccuracy() const;
        const int getRandSeed() const;
        const scoutStrategy getChosenScoutStrategy() const;
        const char *getLogFileName() const;
        const float getSpiralVelocityIncrease() const;
        const unsigned getSpiralAngularVelocityIncrease() const;
        const float getBeeVisionRadius() const;
        const float getSensingNoise() const;
        const float getActuationNoise() const;
        const float getBeeColonySightRadius() const;
        const float getUnemployedScoutProbability() const;
        const unsigned getBeeDanceLengthMultiplier() const;
        const unsigned doSharePositionalInformation() const;
        const unsigned getNumFollowersPerDancer() const;

    private:
        unsigned timeSteps;
        unsigned nectarPerHarvest;
        unsigned beeFlightSpeed;
        unsigned energyToReplenish;
        unsigned numFollowersPerDancer;
        unsigned sharePositionalInformation;
        float spiralVelocityIncrease;
        float beeVisionRadius;
        float beeActuationError;
        float beeSensingError;
        float beeColonySightRadius;
        float unemployedScoutProbability;
        unsigned spiralAngularVelocityIncrease;
        unsigned beeDanceLengthMultiplier;
        float locationAccuracy;
        int randSeed;
        scoutStrategy chosenScoutStrategy;
        char logFileName[50];

};

#endif // SETTINGS_H
