/*
 * Copyright (C) 2010  Peter Bailis <pbailis@eecs.harvard.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <stdlib.h>

#include "ConfigReader.h"
#include "RunSettings.h"
#include "World.h"

using namespace std;

Settings *runSettings;
bool doLog = false;
bool doLogKnownFlowers = false;

//number of times bees have headed to flower location only
//to not find a flower!
unsigned unsuccessfulHeadings = 0;

unsigned totalForageTrips = 0;
unsigned successfulReturnHigh = 0;
unsigned successfulReturnLo = 0;
unsigned unsuccessfulReturns = 0;

int main(int argc, char *argv[])
{
    if(argc < 2 || argc > 3)
    {
        cout << "Usage: beesim <configfile> <opt: debug>" << endl;
        return 1;
    }

    if(argc == 3)
    {
        if(argv[2][0] == 'p')
        {
            doLogKnownFlowers = true;
        }
        else
        {
            doLog = true;
        }
    }

    ConfigReader *reader = new ConfigReader(argv[1]);

    runSettings = reader->getSettings();
    srand(runSettings->getRandSeed());

    World *simWorld = reader->getWorld();

    unsigned totalTime = runSettings->getTimeSteps();
//    unsigned printTime = totalTime / 10;

    for(unsigned time = 0; time < totalTime; ++time)
    {
        simWorld->updateWorld();

        if(false)//!(time % printTime))
        {
            cout << time << endl;
            cout << "N: " << simWorld->totalNectarCollected() << " E: " << simWorld->totalEnergySpent() << endl
            << "Scouting missions: " << simWorld->getNumScouts() << endl
            << "Dances followed: " << simWorld->getDancesFollowed() << endl
            << "Dances offered: " << simWorld->getDancesOffered() << endl
	        << "Forage runs: " << totalForageTrips << endl
            << "Unsuccessful headings: " << unsuccessfulHeadings << endl
            << "Successful low-quality returns: " << successfulReturnLo << endl
            << "Successful high-quality returns: " << successfulReturnHigh << endl;
        }
    }

    cout << "FINAL N: " << simWorld->totalNectarCollected() << " E: " << simWorld->totalEnergySpent() << endl
         << "Scouting missions: " << simWorld->getNumScouts() << endl
         << "Dances followed: " << simWorld->getDancesFollowed() << endl
         << "Dances offered: " << simWorld->getDancesOffered() << endl
         << "Forage runs: " << totalForageTrips << endl
         << "Unsuccessful headings: " << unsuccessfulHeadings << endl
         << "Unsuccessful returns: " << unsuccessfulReturns << endl
         << "Successful low-quality returns: " << successfulReturnLo << endl
         << "Successful high-quality returns: " << successfulReturnHigh << endl;


    delete simWorld;
    delete runSettings;
    delete reader;

    return 0;
}
